@@ -12,7 +12,7 @@ from abc import ABC, abstractmethod
 from time import monotonic
 from types import TracebackType
 from typing import Any, AsyncIterator, cast, Dict, Generic, List
-from typing import Optional, Sequence, Type
+from typing import Optional, Type
 from weakref import ref
 from contextlib import asynccontextmanager
 
@@ -111,7 +111,8 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
             if getattr(self, "_closed", True):
                 return
 
-            self._stop_workers()
+            workers = self._signal_stop_worker()
+            agather(*workers, timeout=5.0)
 
     def _check_open_getconn(self) -> None:
         super()._check_open_getconn()
@@ -125,9 +126,9 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
             else:
                 warnings.warn(
                     f"the default for the {type(self).__name__} 'open' parameter"
-                    + " will become 'False' in a future release. Please use"
-                    + " open={True|False} explicitly, or use the pool as context"
-                    + f" manager using: `with {type(self).__name__}(...) as pool: ...`",
+                    " will become 'False' in a future release. Please use"
+                    " open={True|False} explicitly, or use the pool as context"
+                    f" manager using: `with {type(self).__name__}(...) as pool: ...`",
                     DeprecationWarning,
                 )
 
@@ -139,7 +140,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
                 " is deprecated and will not be supported anymore in a future"
                 " release. Please use `await pool.open()`, or use the pool as context"
                 f" manager using: `async with {type(self).__name__}(...) as pool: `...",
-                DeprecationWarning,
+                RuntimeWarning,
             )
 
     async def wait(self, timeout: float = 30.0) -> None:
@@ -300,7 +301,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
             self._stats[self._REQUESTS_ERRORS] += 1
             raise TooManyRequests(
                 f"the pool {self.name!r} has already"
-                + f" {len(self._waiting)} requests waiting"
+                f" {len(self._waiting)} requests waiting"
             )
         return conn
 
@@ -459,16 +460,24 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
             connections = list(self._pool)
             self._pool.clear()
 
+            # Take the workers out of the pool. Will stop them outside the lock
+            workers = await self._signal_stop_worker()
+
         # Now that the flag _closed is set, getconn will fail immediately,
         # putconn will just close the returned connection.
-        await self._stop_workers(waiting, connections, timeout)
 
-    async def _stop_workers(
-        self,
-        waiting_clients: Sequence[WaitingClient[ACT]] = (),
-        connections: Sequence[ACT] = (),
-        timeout: float | None = None,
-    ) -> None:
+        # Wait for the worker tasks to terminate
+        await agather(*workers, timeout=timeout)
+
+        # Close the connections that were still in the pool
+        for conn in connections:
+            await conn.close()
+
+        # Signal to eventual clients in the queue that business is closed.
+        for pos in waiting:
+            await pos.fail(PoolClosed(f"the pool {self.name!r} is closed"))
+
+    async def _signal_stop_worker(self) -> list[AWorker]:
         # Stop the scheduler
         await self._sched.enter(0, None)
 
@@ -477,18 +486,11 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
         for _ in workers:
             self.run_task(StopWorker(self))
 
-        # Signal to eventual clients in the queue that business is closed.
-        for pos in waiting_clients:
-            await pos.fail(PoolClosed(f"the pool {self.name!r} is closed"))
+        if self._sched_runner:  # likely
+            workers.append(self._sched_runner)
+            self._sched_runner = None
 
-        # Close the connections still in the pool
-        for conn in connections:
-            await conn.close()
-
-        # Wait for the worker tasks to terminate
-        assert self._sched_runner is not None
-        sched_runner, self._sched_runner = self._sched_runner, None
-        await agather(sched_runner, *workers, timeout=timeout)
+        return workers
 
     async def __aenter__(self) -> Self:
         self._open_implicit = False
@@ -567,7 +569,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
         for instance as `!check` callback when a pool is created.
         """
         if conn.autocommit:
-            await conn.execute("SELECT 1")
+            await conn.execute("")
         else:
             if True:  # ASYNC
                 # NOTE: with Psycopg 3.2 we could use conn.set_autocommit() in
@@ -575,13 +577,13 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
                 # previous versions too.
                 await conn.set_autocommit(True)
                 try:
-                    await conn.execute("SELECT 1")
+                    await conn.execute("")
                 finally:
                     await conn.set_autocommit(False)
             else:
                 conn.autocommit = True
                 try:
-                    conn.execute("SELECT 1")
+                    conn.execute("")
                 finally:
                     conn.autocommit = False
 
@@ -658,7 +660,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
                 sname = TransactionStatus(status).name
                 raise e.ProgrammingError(
                     f"connection left in status {sname} by configure function"
-                    + f" {self._configure}: discarded"
+                    f" {self._configure}: discarded"
                 )
 
         # Set an expiry date, with some randomness to avoid mass reconnection
@@ -761,9 +763,26 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
         # Also disable the warning for open connection in conn.__del__
         conn._pool = None
 
+        # Early bailout in case the pool is closed. Don't add anything to the
+        # state. There is still a remote chance that the pool will be closed
+        # between here and entering the lock. Therefore we will make another
+        # check later.
+        if self._closed:
+            await conn.close()
+            return
+
         # Critical section: if there is a client waiting give it the connection
         # otherwise put it back into the pool.
         async with self._lock:
+
+            # Check if the pool was closed by the time we arrived here. It is
+            # unlikely but it doesn't seem impossible, if the worker was adding
+            # this connection while the main process is closing the pool.
+            # Now that we are in the critical section we know for real.
+            if self._closed:
+                await conn.close()
+                return
+
             while self._waiting:
                 # If there is a client waiting (which is still waiting and
                 # hasn't timed out), give it the connection and notify it.
@@ -818,7 +837,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
                     sname = TransactionStatus(status).name
                     raise e.ProgrammingError(
                         f"connection left in status {sname} by reset function"
-                        + f" {self._reset}: discarded"
+                        f" {self._reset}: discarded"
                     )
             except Exception as ex:
                 logger.warning(f"error resetting connection: {ex}")
@@ -841,7 +860,7 @@ class AsyncConnectionPool(Generic[ACT], BasePool):
         if to_close:
             logger.info(
                 "shrinking pool %r to %s because %s unused connections"
-                + " in the last %s sec",
+                " in the last %s sec",
                 self.name,
                 self._nconns,
                 nconns_min,
@@ -962,8 +981,7 @@ class MaintenanceTask(ABC):
         pool.run_task(self)
 
     @abstractmethod
-    async def _run(self, pool: AsyncConnectionPool[Any]) -> None:
-        ...
+    async def _run(self, pool: AsyncConnectionPool[Any]) -> None: ...
 
 
 class StopWorker(MaintenanceTask):
